var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __slice = [].slice;

define(['jquery', 'storage_freak'], function($, StorageFreak) {
  var Selectorablium;
  Selectorablium = (function() {
    Selectorablium.prototype.defaults = {
      minCharsForRemoteSearch: 3,
      localCacheTimeout: 7 * 24 * 60 * 60 * 1000,
      XHRTimeout: 650,
      debounceInterval: 150,
      maxResultsNum: 10,
      default_value: 0,
      default_text: 'Please select an option',
      selected_id: null,
      list_of_replacable_chars: [['ά', 'α'], ['έ', 'ε'], ['ή', 'η'], ['ί', 'ι'], ['ό', 'ο'], ['ύ', 'υ'], ['ώ', 'ω']]
    };

    Selectorablium.prototype._required = ['app_name', 'url', 'query', 'name', 'minCharsForRemoteSearch', 'XHRTimeout', 'list_of_replacable_chars', 'localCacheTimeout'];

    Selectorablium.prototype.template = "<div class=\"selectorablium_cont\">\n  <div class=\"top\">\n    <div class=\"initial_loader\">Loading initial data...</div>\n    <a class=\"clear_button\">Clear</a>\n  </div>\n  <div class=\"inner_container\">\n    <form>\n      <input autocomplete=\"off\" name=\"var_name\">\n      <span class=\"input_icon\"></span>\n      <div class=\"loader\"></div>\n      <div class=\"XHRCounter\"></div>\n    </form>\n    <ul class=\"list_container\"></ul>\n  </div>\n</div>";

    Selectorablium.prototype.timer = 0;

    function Selectorablium(element, options) {
      this._onDBSearchResults = __bind(this._onDBSearchResults, this);
      this._onDBSearchReset = __bind(this._onDBSearchReset, this);
      this._onDBSearchError = __bind(this._onDBSearchError, this);
      this._onDBSearchEnd = __bind(this._onDBSearchEnd, this);
      this._onDBSearchStart = __bind(this._onDBSearchStart, this);
      this._onDBSearchIn = __bind(this._onDBSearchIn, this);
      this._onDBCreateEnd = __bind(this._onDBCreateEnd, this);
      this._onDBCreateStart = __bind(this._onDBCreateStart, this);
      this._onDBReady = __bind(this._onDBReady, this);
      this._onKeyUp = __bind(this._onKeyUp, this);
      this._onKeyPress = __bind(this._onKeyPress, this);
      this._activateSelectedItem = __bind(this._activateSelectedItem, this);
      this._onItemMouseEnter = __bind(this._onItemMouseEnter, this);
      this._onHTMLClick = __bind(this._onHTMLClick, this);
      this._clearSelectedItem = __bind(this._clearSelectedItem, this);
      this._onClearButtonClick = __bind(this._onClearButtonClick, this);
      this._onContainerClick = __bind(this._onContainerClick, this);
      this._togglePlugin = __bind(this._togglePlugin, this);
      var attr, _i, _len, _ref;
      if (this instanceof Selectorablium === false) {
        return new Selectorablium(element, options);
      }
      if (!element) {
        throw new Error('Element argument is required');
      }
      this.$el = $(element);
      this.options = options;
      this.metadata = this.$el.data();
      this.config = $.extend({}, this.defaults, this.metadata, this.options);
      _ref = this._required;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        attr = _ref[_i];
        if (!this.config[attr]) {
          throw new Error("'" + attr + "' option is required");
        }
      }
      this.state = 'disabled';
      this.shift_pressed = false;
      this.structure = [];
      this.indexed_structure = {};
      this.$result_items = [];
      this.events_handled = [];
      this.db = new StorageFreak(this.config);
      this._createHtmlElements();
      this._registerEventHandlers();
      this.data_ready_dfd = this.db.init();
      this.data_ready_dfd.then(this._onDBReady);
    }

    Selectorablium.prototype.cleanup = function() {
      var item, _i, _len, _ref;
      this.db.cleanup();
      _ref = this.events_handled;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        item[0].off.apply(item[0], item[1]);
      }
      this.$container.remove();
      this.$el.unwrap();
      return this.$outer_container.remove();
    };

    Selectorablium.prototype.then = function(success, fail) {
      return this.data_ready_dfd.then(success, fail);
    };

    Selectorablium.prototype.reset = function() {
      return this._insertOptionElement(this.config.default_value, this.config.default_text);
    };

    Selectorablium.prototype.set = function(value, text) {
      if (text == null) {
        text = null;
      }
      if (text === null) {
        text = this.db.searchByValue(value);
        if (text === false) {
          return false;
        }
      }
      this._insertOptionElement(value, text);
      this._hide();
      return true;
    };

    Selectorablium.prototype.add = function(value, text) {
      return this.db.add(value, text);
    };

    Selectorablium.prototype._createHtmlElements = function() {
      this.$el.wrap($('<div class="selectorablium_outer_cont">'));
      this.$outer_container = this.$el.parent();
      this.$outer_container.append(this.template);
      this.$container = this.$outer_container.find('.selectorablium_cont');
      this.$top = this.$container.find('.top');
      this.$top_loader = this.$container.find('.initial_loader');
      this.$inner_container = this.$container.find('.inner_container');
      this.$input = this.$container.find('input');
      this.$results_list = this.$container.find('.list_container');
      this.$XHR_counter = this.$container.find('.XHRCounter');
      this.$XHR_loader = this.$container.find('.loader');
      this.$clear_button = this.$container.find('.clear_button');
      return this.reset();
    };

    Selectorablium.prototype._registerEventHandlers = function() {
      this._addHandler(this.$top, 'click', this._togglePlugin);
      this._addHandler(this.$el, 'focus', this._togglePlugin);
      this._addHandler(this.$container, 'click', this._onContainerClick);
      this._addHandler(this.$clear_button, 'click', this._onClearButtonClick);
      this._addHandler($('html'), 'click', this._onHTMLClick);
      this._addHandler(this.$input, 'keyup', this._onKeyUp);
      this._addHandler(this.$input, this._keyPressEventName(), this._onKeyPress);
      this._addHandler(this.$results_list, 'mouseenter', '.item', this._onItemMouseEnter);
      this._addHandler(this.$results_list, 'click', '.item', this._activateSelectedItem);
      this.db.on('dbcreate_start', this._onDBCreateStart);
      this.db.on('dbcreate_end', this._onDBCreateEnd);
      this.db.on('dbremote_search_in', this._onDBSearchIn);
      this.db.on('dbremote_search_start', this._onDBSearchStart);
      this.db.on('dbremote_search_end', this._onDBSearchEnd);
      this.db.on('dbremote_search_error', this._onDBSearchError);
      this.db.on('dbremote_search_reset', this._onDBSearchReset);
      return this.db.on('dbsearch_results', this._onDBSearchResults);
    };

    Selectorablium.prototype._togglePlugin = function(e) {
      switch (this.state) {
        case 'disabled':
          break;
        case 'visible':
          return this._hide();
        case 'ready':
          if (this.$inner_container.is(':visible') === false) {
            return this._show();
          }
      }
    };

    Selectorablium.prototype._onContainerClick = function(e) {
      return this.do_not_hide_me = true;
    };

    Selectorablium.prototype._onClearButtonClick = function(e) {
      this._clearSelectedItem(e);
      return false;
    };

    Selectorablium.prototype._clearSelectedItem = function(e) {
      this.reset();
      this._hide();
      return this.$el.trigger("change");
    };

    Selectorablium.prototype._onHTMLClick = function(e) {
      if (this.do_not_hide_me === true) {
        return this.do_not_hide_me = false;
      } else {
        return this._hide();
      }
    };

    Selectorablium.prototype._onItemMouseEnter = function(e) {
      return this._selectItem($(e.currentTarget));
    };

    Selectorablium.prototype._activateSelectedItem = function(e) {
      if (this.selected_item) {
        this._insertOptionElement(this.selected_item.data('value'), this.selected_item.text());
        this.$el.trigger("change");
      }
      this._hide();
      return false;
    };

    Selectorablium.prototype._onKeyPress = function(e) {
      switch (e.keyCode) {
        case 16:
          this.shift_pressed = true;
          break;
        case 27:
          this._hide();
          break;
        case 38:
          this._moveSelectedItem('up');
          break;
        case 40:
          this._moveSelectedItem('down');
          break;
        case 13:
          this._activateSelectedItem();
          break;
        case 9:
          this._activateSelectedItem();
          return true;
        default:
          return true;
      }
      return false;
    };

    Selectorablium.prototype._onKeyUp = function(e) {
      var search_func;
      switch (e.keyCode) {
        case 17:
        case 18:
        case 37:
        case 38:
        case 39:
        case 40:
        case 27:
        case 13:
        case 91:
        case 20:
        case 33:
        case 34:
        case 35:
        case 36:
        case 45:
        case 9:
          return false;
        case 16:
          this.shift_pressed = false;
          return false;
      }
      this._resetContainer();
      this.selected_item = null;
      this.query = $.trim(this.$input.val());
      if (this.query.length === 0) {
        this._resetSelectedItem();
        return false;
      }
      search_func = (function(_this) {
        return function() {
          return _this.db.search(_this.query);
        };
      })(this);
      clearTimeout(this.timer);
      this.timer = setTimeout(search_func, this.config.debounceInterval);
      return false;
    };

    Selectorablium.prototype._onDBReady = function(data) {
      if (this.config.selected_id) {
        this.set(this.config.selected_id);
      }
      return this.state = 'ready';
    };

    Selectorablium.prototype._onDBCreateStart = function() {
      this.state = 'disabled';
      this.$top_loader.show();
      return this.$top.addClass('disabled');
    };

    Selectorablium.prototype._onDBCreateEnd = function() {
      this.$top_loader.hide();
      return this.$top.removeClass('disabled');
    };

    Selectorablium.prototype._onDBSearchIn = function(time) {
      return this._showXHRCountdown(time);
    };

    Selectorablium.prototype._onDBSearchStart = function() {
      this._hideXHRCountdown();
      return this.$XHR_loader.show();
    };

    Selectorablium.prototype._onDBSearchEnd = function() {
      return this.$XHR_loader.hide();
    };

    Selectorablium.prototype._onDBSearchError = function(xhr) {
      return this.$XHR_loader.hide();
    };

    Selectorablium.prototype._onDBSearchReset = function() {
      this._hideXHRCountdown();
      return this.$XHR_loader.hide();
    };

    Selectorablium.prototype._onDBSearchResults = function(results, query, xhr) {
      var selected_item;
      if (xhr == null) {
        xhr = false;
      }
      if (query !== this.query) {
        return;
      }
      this._updateStructure(results, xhr);
      this._highlightResults(query);
      this.$result_items = this._appendResults(query);
      this.$result_items.filter('.new').hide().slideToggle();
      if ((selected_item = this.$result_items.filter('.selected')).length === 0) {
        selected_item = this.$result_items.first();
      }
      return this._selectItem(selected_item);
    };

    Selectorablium.prototype._updateStructure = function(results, xhr) {
      var key, new_item, new_keys, result, value, _i, _len, _ref;
      this.structure = [];
      new_keys = {};
      for (_i = 0, _len = results.length; _i < _len; _i++) {
        result = results[_i];
        new_keys[result.name] = true;
        new_item = {
          id: result.id,
          name: result.name,
          template: '',
          selected: false,
          fresh: !!xhr
        };
        if (this.indexed_structure[result.name]) {
          new_item = $.extend(new_item, this.indexed_structure[result.name]);
        }
        this.indexed_structure[result.name] = new_item;
        this.structure.push(new_item);
      }
      _ref = this.indexed_structure;
      for (key in _ref) {
        value = _ref[key];
        if (!new_keys[key]) {
          this.indexed_structure[key] = null;
          delete this.indexed_structure[key];
        }
      }
      return this.structure;
    };

    Selectorablium.prototype._highlightResults = function(query) {
      var item, re, _i, _len, _ref, _results;
      re = this._createAccentIndependentRE(query);
      _ref = this.structure;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        _results.push(item.template = item.name.replace(re, '<span class="highlight">$1</span>'));
      }
      return _results;
    };

    Selectorablium.prototype._appendResults = function(query) {
      this.$results_list.empty();
      this.$results_list.append(this._createTemplate(query));
      return this.$results_list.find('.item');
    };

    Selectorablium.prototype._createTemplate = function(query) {
      var class_name, item, items_templates, text, _i, _len, _ref;
      items_templates = [];
      if (this.structure.length === 0) {
        text = 'No results found';
        items_templates.push("<li><p class='empty-message'>" + text + "</p></li>");
      } else {
        _ref = this.structure;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          item = _ref[_i];
          class_name = 'item';
          if (item.fresh === true) {
            class_name += ' new';
          }
          if (item.selected === true) {
            class_name += ' selected';
          }
          items_templates.push("<li><a href='#' class='" + class_name + "' data-value='" + item.id + "' data-text='" + item.name + "'>" + item.template + "</a></li>");
          item.fresh = false;
        }
      }
      return items_templates.join('\n');
    };

    Selectorablium.prototype._createAccentIndependentRE = function(query) {
      var re, value, _i, _len, _ref;
      _ref = this.config.list_of_replacable_chars;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        value = _ref[_i];
        re = new RegExp("" + value[0] + "|" + value[1], 'ig');
        query = query.replace(re, "(?:" + value[0] + "|" + value[1] + ")");
      }
      re = null;
      return new RegExp("(" + query + ")", 'ig');
    };

    Selectorablium.prototype._addHandler = function() {
      var $element, on_args;
      $element = arguments[0], on_args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      $element.on.apply($element, on_args);
      return this.events_handled.push([$element, on_args]);
    };

    Selectorablium.prototype._keyPressEventName = function() {
      var ff_ua_match;
      ff_ua_match = window.navigator.userAgent.match(/Firefox\/\d+\.\d+\.\d+/);
      if (window.opera || !!ff_ua_match) {
        return 'keypress';
      } else {
        return 'keydown';
      }
    };

    Selectorablium.prototype._insertOptionElement = function(value, text) {
      return this.$el.html("<option value=\"" + value + "\">" + text + "</option>");
    };

    Selectorablium.prototype._showXHRCountdown = function(milliseconds) {
      this.$XHR_counter.stop(true, true).css('width', '0').show();
      return this.$XHR_counter.animate({
        width: '100%'
      }, milliseconds, function() {
        return $(this).hide();
      });
    };

    Selectorablium.prototype._hideXHRCountdown = function() {
      return this.$XHR_counter.stop(true, true).hide();
    };

    Selectorablium.prototype._show = function() {
      this.$container.addClass('active');
      this.$el.addClass('active');
      this.$inner_container.slideDown(200);
      this.$input.focus();
      return this.state = 'visible';
    };

    Selectorablium.prototype._hide = function() {
      if (this.state !== 'visible') {
        return;
      }
      this.$container.removeClass('active');
      this.$el.removeClass('active');
      this.$inner_container.slideUp(200);
      this._resetSelectedItem();
      this._resetContainer();
      this.$input.val('');
      this.query = '';
      return this.state = 'ready';
    };

    Selectorablium.prototype._resetContainer = function() {
      this.$results_list.empty();
      this.$XHR_loader.hide();
      return this._hideXHRCountdown();
    };

    Selectorablium.prototype._resetSelectedItem = function() {
      var ref;
      if (!this.selected_item) {
        return;
      }
      ref = this.indexed_structure[this.selected_item.data('text')];
      ref && (ref.selected = false);
      this.selected_item.removeClass('selected');
      return this.selected_item = null;
    };

    Selectorablium.prototype._selectItem = function($item) {
      if (!($item && $item.length !== 0)) {
        return;
      }
      this._resetSelectedItem();
      this.selected_item = $item.addClass('selected');
      return this.indexed_structure[$item.data('text')].selected = true;
    };

    Selectorablium.prototype._getNextItem = function(direction) {
      var count, custom_index, index, new_index;
      count = this.$result_items.length;
      index = this.$result_items.index(this.selected_item) + count;
      custom_index = direction === 'up' ? index - 1 : index + 1;
      new_index = custom_index % count;
      return $(this.$result_items.get(new_index));
    };

    Selectorablium.prototype._moveSelectedItem = function(direction) {
      var $item;
      if (!this.selected_item) {
        return;
      }
      $item = this._getNextItem(direction);
      return this._selectItem($item);
    };

    return Selectorablium;

  })();
  return Selectorablium;
});
